/* Automatically generated file.  Do not edit */
#define OP_Goto                          1
#define OP_Gosub                         2
#define OP_Return                        3
#define OP_Halt                          4
#define OP_Integer                       5
#define OP_Real                          6
#define OP_String8                       7
#define OP_String                        8
#define OP_HexBlob                       9
#define OP_Blob                         10
#define OP_Variable                     11
#define OP_Pop                          12
#define OP_Dup                          13
#define OP_Pull                         14
#define OP_Push                         15
#define OP_Callback                     16
#define OP_Concat8                      17
#define OP_Concat                       18
#define OP_Add                          19
#define OP_Subtract                     20
#define OP_Multiply                     21
#define OP_Divide                       22
#define OP_Remainder                    23
#define OP_CollSeq                      24
#define OP_Function                     25
#define OP_BitAnd                       26
#define OP_BitOr                        27
#define OP_ShiftLeft                    28
#define OP_ShiftRight                   29
#define OP_AddImm                       30
#define OP_ForceInt                     31
#define OP_MustBeInt                    32
#define OP_Eq                           33
#define OP_Ne                           34
#define OP_Lt                           35
#define OP_Le                           36
#define OP_Gt                           37
#define OP_Ge                           38
#define OP_And                          39
#define OP_Or                           40
#define OP_Negative                     41
#define OP_AbsValue                     42
#define OP_Not                          43
#define OP_BitNot                       44
#define OP_Noop                         45
#define OP_If                           46
#define OP_IfNot                        47
#define OP_IsNull                       48
#define OP_NotNull                      49
#define OP_SetNumColumns                50
#define OP_IdxColumn                    51
#define OP_Column                       52
#define OP_MakeRecord                   53
#define OP_Statement                    54
#define OP_AutoCommit                   55
#define OP_Transaction                  56
#define OP_ReadCookie                   57
#define OP_SetCookie                    58
#define OP_VerifyCookie                 59
#define OP_OpenRead                     60
#define OP_OpenWrite                    61
#define OP_OpenTemp                     62
#define OP_OpenPseudo                   63
#define OP_Close                        64
#define OP_MoveLt                       65
#define OP_MoveLe                       66
#define OP_MoveGe                       67
#define OP_MoveGt                       68
#define OP_Distinct                     69
#define OP_NotFound                     70
#define OP_Found                        71
#define OP_IsUnique                     72
#define OP_NotExists                    73
#define OP_NewRecno                     74
#define OP_PutIntKey                    75
#define OP_PutStrKey                    76
#define OP_Delete                       77
#define OP_ResetCount                   78
#define OP_KeyAsData                    79
#define OP_RowKey                       80
#define OP_RowData                      81
#define OP_Recno                        82
#define OP_FullKey                      83
#define OP_NullRow                      84
#define OP_Last                         85
#define OP_Rewind                       86
#define OP_Prev                         87
#define OP_Next                         88
#define OP_IdxPut                       89
#define OP_IdxDelete                    90
#define OP_IdxRecno                     91
#define OP_IdxLT                        92
#define OP_IdxGT                        93
#define OP_IdxGE                        94
#define OP_IdxIsNull                    95
#define OP_Destroy                      96
#define OP_Clear                        97
#define OP_CreateIndex                  98
#define OP_CreateTable                  99
#define OP_ParseSchema                 100
#define OP_DropTable                   101
#define OP_DropIndex                   102
#define OP_DropTrigger                 103
#define OP_IntegrityCk                 104
#define OP_ListWrite                   105
#define OP_ListRewind                  106
#define OP_ListRead                    107
#define OP_ListReset                   108
#define OP_ListPush                    109
#define OP_ListPop                     110
#define OP_ContextPush                 111
#define OP_ContextPop                  112
#define OP_SortPut                     113
#define OP_Sort                        114
#define OP_SortNext                    115
#define OP_SortReset                   116
#define OP_MemStore                    117
#define OP_MemLoad                     118
#define OP_MemIncr                     119
#define OP_AggReset                    120
#define OP_AggInit                     121
#define OP_AggFunc                     122
#define OP_AggFocus                    123
#define OP_AggSet                      124
#define OP_AggGet                      125
#define OP_AggNext                     126
#define OP_Vacuum                      127
